const chalk = require('chalk');
const copy = require('fs-extra').copy;

const isObj = (obj) => {
  if (
    typeof obj === 'object' &&
    Object.prototype.toString.call(obj).toLowerCase() === '[object object]'
  ) {
    return true;
  }
  return false;
};

class CopyWebpackOutputPlugin {
  constructor(patterns, options) {
    this.pluginName = 'copy-webpack-output-plugin';

    patterns = typeof patterns === 'undefined' ? [] : patterns;

    if (typeof options !== 'undefined' && !isObj(options)) {
      console.log(chalk.red(this.pluginName) + ' options must be a object');
      return;
    }

    patterns.map((item) => {
      if (item.hasOwnProperty('options') && !isObj(item.options)) {
        console.log(
          chalk.red(this.pluginName) + ' options in patterns must be a object'
        );
        return;
      }
      item.options = Object.assign({}, options, item.options || {});
    });

    this.patterns = patterns;
  }

  apply(compiler) {
    if (typeof compiler === 'undefined') {
      this.copy(compiler);
      return;
    }

    compiler.hooks.done.tap('CopyWebpackOutputPlugin', () => {
      this.copy(compiler);
    });
  }

  copy(compiler) {
    this.patterns.forEach((item) => {
      copy(item.src, item.dest, this.options)
        .then(() => {
          console.log(
            'copy-webpack-output-plugin copy ' +
              chalk.bold.green(item.src) +
              ' to ' +
              chalk.bold.green(item.dest) +
              ' success'
          );
        })
        .catch((err) => {
          throw err;
        });
    });
  }
}

module.exports = CopyWebpackOutputPlugin;
